/**
 * ===========================================================================
 *                      Backtracking: COMBINATIONS
 * Combinations are dispositions, where the order is NOT significant.
 * In the dispositions the order is significant. The combinations of 'B over D'
 * are the different SETS (not sequences) of the B elements taken in groups of
 * cardinality D. The B elements are assumed to be: 1,...,B.
 * The combinations will be presented in lexicographic order, that is,
 * X1,...,XD comes before Y1,...,YD iff there exists k, with 0<=k<D, such that
 * X1 = Y1, ..., Xk = Yk, Xk+1 < Yk+1.
 * ===========================================================================
 */

public class Combinations
{
 private int id = 0;
 private final int  B = 4;              // B should be positive
 private final int  D = 2;              // D should be positive and D<=B
 private int[] arrayA = new int[D];     // by default: initially 0

 private void printar(int[] arrayA, int size) {
    for (int k=0; k<size; k++) {System.out.print(arrayA[k]+" ");}
    System.out.println();}

 private void generate() {
    int ib=1;
    do {// -------- For id==0 the first element is always ok.
        // -------- 'increasing' dispositions because of (ib > A[id-1]) below.
        if ((id == 0) || (ib > arrayA[id-1]))
           {arrayA[id] = ib; id++;
           if (id == D) {printar(arrayA,D);} else generate();
           id--;};
        ib++;}
    while (ib<=B);
  }

 public static void main(String[] args) {

   Combinations comb = new Combinations();
   comb.generate();
 }
}
/**
 * input:                   output:
 * ---------------------------------------------------------------------------
 * javac Combinations.java
 * java  Combinations
 *                          1 2
 *                          1 3
 *                          1 4
 *                          2 3
 *                          2 4
 *                          3 4
 * ---------------------------------------------------------------------------
 */
