/**
 * ===========================================================================
 *                       Backtracking: n QUEENS
 * From: Timothy Budd: Understanding Object-Oriented Programming with Java.
 * Addison-Wesley, 2000. Updated Edition. With changes by A. Pettorossi. 
 * This program is executed by typing the command line (n should be >= 1):
 *    java QueensSimpleSolver n   
 * The value of n is the number of queens, denoted by NQ. 
 * The queens are placed from top to bottom, one for each row.
 *
 * A queen is a 4-tuple <queens_number,row,column,nextqueen>.
 * nextqueen (called neighbour) is a queen: it is a pointer to the next queen.
 * It is difficult to show the correctness of the algorithm. For instance:
 * - if we place instruction (3) at point (4) we get a wrong program.
 * - line (2) (see Budd's book) seems to be unnecessary. (I did not prove it).
 *
 * attack(r,c) checks that no queen that is in the list outgoing
 * from "this one", is attacking <r,c>. Nothing is said about the
 * relative attack among the queens in that list. This is why in line (3)
 * we construct the solution for each new queen which is placed in that list.
 * Line (0) can be replaced by:
 *          { if (! advance()) return false; };
 * Line (1) can be replaced by:
 *          if (! neighbour.advance()) return false;
 * ---------------------------------------------------------------------------
 * Object oriented programming is NOT always well suited for recursive
 * programming. 
 *
 * THIS PROGRAM IS MUCH SLOWER THAN DIJKSTRA'S PROGRAM (try for NQ=25 or 26).
 * ===========================================================================
 */

  class Queen {                 // data fields
        private int nqueens;
        private int row;
        private int column;
        private Queen neighbour;// Queen = <queens_number,row,column> x Queen

                                // constructor
        Queen (int nqueens, int row, int column, Queen neighbour) {
                                // initialize data fields
                this.nqueens = nqueens; 
                this.row = row; 
                this.column = column;
                this.neighbour = neighbour;}

        public boolean constructSolution() {
           if (neighbour != null && neighbour.attacks(row, column))
               { boolean next = advance(); if (! next) return false; };  //(0)
           return true;
           }

        public boolean advance() {  // has side effects: column is changed !
           if (column < nqueens) {column++; return constructSolution(); } else
           if (neighbour != null) {
             boolean next = neighbour.advance(); if (!next) return false;//(1)
             // if (! neighbour.constructSolution()) return false;       //(2)
             }
           else return false;       // neighbour == null
        /**
         * After executing a return statement no other statements of  
         * the method are executed 
         */
           column = 1; 
           return constructSolution();
           }

        private boolean attacks(int aRow, int aColumn) {
           int rowDifference = aRow - row;
           if ((column == aColumn) ||
               (column + rowDifference == aColumn) ||
               (column - rowDifference == aColumn))
                           {return true;}
                if (neighbour == null)
                           {return false;}
                return neighbour.attacks(aRow, aColumn);
                }

        public void print() {
           if (neighbour != null) neighbour.print();
           System.out.print(column + " ");
           }
  }

  public class QueensSimpleSolver {

        public static void main(String [ ] args) {

           int ns = 0;        // number of solutions
           boolean sol;       // sol == true if a new solution is found
           int NQ = Integer.parseInt(args[0]);

           if ( NQ <= 0 )
           { throw new RuntimeException("\nQueens error: illegal input!"); }
           System.out.println("\nThere are "+ NQ +" queens. The solutions " + 
                              "are:");

           int row=1; Queen lastQueen = null;
           do { lastQueen = new Queen(NQ, row, 1, lastQueen);
                sol = lastQueen.constructSolution();             //(3)
                row++;
              }
           while (row<=NQ);
           // -------------------------------------------------  //(4)

           // Computing the next solution. No more solutions if sol == false
           while (sol) {ns++ ;
                        System.out.println(); 
                        lastQueen.print();
                        sol = lastQueen.advance();};
           System.out.println("\n\nNumber of solutions = "+ns);
           }
  }
/**
 * input:                         output:
 * ---------------------------------------------------------------------------
 * javac QueensSimpleSolver.java
 * Java  QueensSimpleSolver 6
 *                                There are 6 queens. The solutions are:
 *
 *                                2 4 6 1 3 5
 *                                3 6 2 5 1 4
 *                                4 1 5 2 6 3
 *                                5 3 1 6 4 2
 *
 *                                Number of solutions = 4
 * ---------------------------------------------------------------------------
 */
