/**
 * ===========================================================================
 *                       Backtracking: n QUEENS.  GRAPHIC SOLVER.
 * From: Timothy Budd. With changes by Alberto Pettorossi. December 2003.
 *
 * public class QueensSolver
 * NQ is the number of queens. It should be: NQ >= 1.
 * The program is executed by typing the command line:
 * java QueensSolver n   (with the number of queens n >= 1)
 * ===========================================================================
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class Queen {
                // data fields
        private int nqueens; private int row; private int column;
        private Queen neighbour;// Queen = <queens_number,row,column> x Queen

                // constructor
        Queen (int nqueens, int row, int column, Queen neighbour) {
                        // initialize data fields
               this.nqueens = nqueens;
               this.row = row;
               this.column = column;
               this.neighbour = neighbour;
               }

        public boolean constructSolution() {
                while (neighbour != null && neighbour.attachs(row, column))
                        if (! advance()) return false;
                return true;
                }

        public boolean advance() {
                if (column < nqueens) {
                        column++;
                        return constructSolution();
                        }
                if (neighbour != null) {
                        if (! neighbour.advance())  return false;
                        if (! neighbour.constructSolution()) return false;
                        }
                else return false;
                column = 1;
                return constructSolution();

                }

        private boolean attachs(int testRow, int testColumn) {
                int rowDifference = testRow - row;
                if ((column == testColumn) ||
                        (column + rowDifference == testColumn) ||
                        (column - rowDifference == testColumn))
                           return true;
                if (neighbour != null)
                           return neighbour.attachs(testRow, testColumn);
                return false;
                }

        public void paint (Graphics g) {
                        // first draw neighbour
                if (neighbour != null)  neighbour.paint(g);
                        // then draw ourself.  x, y is upper left corner
                int x = (column - 1) * 50 + 20;  // x is for columns
                int y = (row - 1) * 50 + 80;     // y is for rows
                g.fillOval(x+15, y+15, 20, 20);
                }
}

public class QueensSolver extends JFrame {
        private int NQ = 0;
        private int ns = 0;  // number of solutions
                             // newsol == true is new solution is found
        private boolean newsol=false;

        public static void main(String [ ] args) {
                int NQ = Integer.parseInt(args[0]);
                if ( NQ <= 0 ) {
                throw new RuntimeException("\nQueens error: illegal input");};
                QueensSolver board = new QueensSolver(NQ);
                board.show();
        }

        private Queen lastQueen = null;  // initialization of queen

        public QueensSolver(int NQ) {
                this.NQ = NQ;
                setTitle(NQ + " queens");
                setSize(200+50*NQ, 100+50*NQ);
                for (int r = 1; r <= NQ; r++) {
                        lastQueen = new Queen(NQ, r, 1, lastQueen);
                        newsol = lastQueen.constructSolution();
                        };
                if (newsol) {ns++;};
                addMouseListener(new MouseClick());
                addWindowListener(new Quit());
                }

        public void paint(Graphics g) {
                super.paint(g);
                                                // draw board
                for (int i = 0; i <= NQ; i++) {
                        g.drawLine(50*i + 20, 80, 50*i + 20, 50*NQ + 80);
                        g.drawLine(20, 50*i + 80, 20 + 50*NQ, 50*i + 80);
                }
                if (newsol)
                   {g.drawString("Computed solutions so far: "+ns, 20, 40);
                    lastQueen.paint(g);       // draw queens
                    g.drawString("Click Mouse for Next Solution", 20, 70);}
                else {g.drawString("No more solutions!", 20, 55);
                      g.drawString("Total number of solutions: "+ns,20, 70);};
                }

        private class Quit extends WindowAdapter {
                public void windowClosing (WindowEvent e) {
                        System.exit(0);
                }
        }

        private class MouseClick extends MouseAdapter {
                public void mousePressed (MouseEvent e) {
                      newsol = lastQueen.advance();
                      if (newsol) ns++;
                      repaint();
               };
        }
}
/**
 * input:                         output:
 * ---------------------------------------------------------------------------
 * javac QueensSimpleSolver.java
 * Java  QueensSimpleSolver 6
 *                                (see figures below)
 * ---------------------------------------------------------------------------
 */

