/**
 * ==========================================================================
 *                  Bounded Buffer controlled by semaphores
 * 
 * It is called by the method ProducerConsumerMain
 * It uses BinarySemaphore.java and CountingSemaphore.java
 *
 * The commands with  // statement for tracing 
 * make visible the insertiona and deletion of elements in the buffer.
 * ==========================================================================
 */

public class BoundedBuffer {
   final int n = 3;     // size of buffer
   float [] buffer = new float[n];
   int in = 0, out = 0;
   BinarySemaphore mutex     = new BinarySemaphore(true);
   CountingSemaphore isEmpty = new CountingSemaphore(0);
   CountingSemaphore isFull  = new CountingSemaphore(n);

   public void put(float value) {
      isFull.Wait();
      mutex.Wait();
      buffer[in] = value;
      System.out.println("-> " + value);         // statement for tracing
      in = (in + 1) % n;
      mutex.Signal();
      isEmpty.Signal();
   }

   public float get() {
      float value;
      isEmpty.Wait();
      mutex.Wait();
      value = buffer[out];
      System.out.println("   " + value + " ->"); // statement for tracing
      out = (out + 1) % n;
      mutex.Signal();
      isFull.Signal();
      return value;
   }
}
