/**
 * ==========================================================================
 *                  Producer and Consumer classes 
 *
 * They use a bounded buffer whose access is controlled by SEMAPHORES.
 * ==========================================================================
 */

class Producer implements Runnable {
   BoundedBuffer b = null;

   public Producer(BoundedBuffer b) {
      this.b = b;
      new Thread(this).start();
   }

   public void run() {
      float item = 0;
      while (true) {
         item++;
         b.put(item);
         try { Thread.sleep(10);
         } catch (InterruptedException e) {}
      }
   }
}

class Consumer implements Runnable {
   BoundedBuffer b = null;

   public Consumer(BoundedBuffer b) {
      this.b = b;
      new Thread(this).start();
   }

   public void run() {
      float item;
      while (true) {
         item = b.get();
         try { Thread.sleep(100);
         } catch (InterruptedException e) {}
      }
   }
}

public class ProducerConsumerSemaphoresMain {
  public static void main(String[] args) {
    BoundedBuffer buffer = new BoundedBuffer();
    Producer producer = new Producer(buffer);
    Consumer consumer = new Consumer(buffer);
  }
}
/** 
 * The output is for a bounded buffer of size 3.
 * input:                               output:
 * ---------------------------------------------------------------------------
 * javac ProducerConsumerSemaphoresMain.java
 * java  ProducerConsumerSemaphoresMain
 *                                      -> 1.0
 *                                         1.0 ->
 *                                      -> 2.0
 *                                      -> 3.0
 *                                      -> 4.0
 *                                         2.0 ->
 *                                      -> 5.0
 *                                         3.0 ->
 *                                      -> 6.0
 *                                         4.0 ->
 *                                      -> 7.0
 *                                         5.0 ->
 *                                      -> 8.0
 *                                      ...
 * ---------------------------------------------------------------------------
 */
