/**
 * ==========================================================================
 *                  Producer and Consumer classes 
 *
 * They use a bounded buffer whose access is controlled by a MONITOR.
 * ==========================================================================
 */

class Producer implements Runnable {
   BoundedBufferMonitor b = null;

   public Producer(BoundedBufferMonitor b) {
      this.b = b;
      new Thread(this).start();
   }

   public void run() {
      float item = 0;
      while (true) {
         item++;
         b.put(item);
         try { Thread.sleep(10);
         } catch (InterruptedException e) {}
      }
   }
}

class Consumer implements Runnable {
   BoundedBufferMonitor b = null;

   public Consumer(BoundedBufferMonitor b) {
      this.b = b;
      new Thread(this).start();
   }

   public void run() {
      float item;
      while (true) {
         item = b.get();
         try { Thread.sleep(100);
         } catch (InterruptedException e) {}
      }
   }
}


public class ProducerConsumerMonitorMain {
  public static void main(String[] args) {
    BoundedBufferMonitor buffer = new BoundedBufferMonitor();
    Producer producer = new Producer(buffer);
    Consumer consumer = new Consumer(buffer);
  }
}

/** 
 * The output is for a bounded buffer of size 3.
 * input:                             output:
 * ---------------------------------------------------------------------------
 * javac ProducerConsumerMonitorMain.java
 * java  ProducerConsumerMonitorMain
 *                                    -> 1.0
 *                                       1.0 ->
 *                                    -> 2.0
 *                                    -> 3.0
 *                                    -> 4.0
 *                                       2.0 ->
 *                                    -> 5.0
 *                                       3.0 ->
 *                                     -> 6.0
 *                                       4.0 ->
 *                                    -> 7.0
 *                                       5.0 ->
 *                                    -> 8.0
 *                                    ...
 * ---------------------------------------------------------------------------
 */
