/**
 * ===========================================================================
 *                       CONCURRENT QUICKSORT
 * 
 * It randomly generates an array of n elements to be sorted.
 * n should be >= 1. If n is not supplied, then we assume that n=20. 
 *
 * It uses QuickSortThread and Maximum.  Thanks to Prof. Marco Danelutto.
 * ===========================================================================
 */

import java.util.Random;
import Maximum.*;

public class ConcurQuickSort {

    public static void main (String[] args) {

        int n; 
        final int MAX = 80;
        try {
            n = Integer.parseInt(args[0]);
        } catch (Exception e) { 
        System.out.print("\nType: java ConcurQuickSort n (with n>=1). ");
        System.out.print("Now we have assumed n=20.\n");
            n = 20;
        }
                       // constructing an array of n elements
        int[] array = new int [n];

                       // initializing the array with random integers
        Random generator = new Random();
        for(int i=0;i<n;i++){
            array[i] = generator.nextInt(MAX);

        }              // printing the initial array
        System.out.println("");
        for(int i=0;i<n;i++) {
            System.out.print(array[i]+" ");
        }
        System.out.println("\n");

        Maximum max = new Maximum();

                       // the initial thread is "go"
        Thread go = new QuickSortThread(array,0,n-1,max);
        go.start();  
                       // waiting the end of initial thread             
        try {
            go.join(); 
        } catch(InterruptedException e) {}

                       // printing the ordered array
        for(int i=0;i<n;i++) {
            System.out.print(array[i]+" ");
        }

                       // printing the maximum number of active threads
        System.out.print("\n\nAt most "+max.get());
        System.out.println(" threads active at the same time.");
        }
}

/** 
 * input:                       output:
 * ---------------------------------------------------------------------------
 * javac ConcurQuickSort.java
 * java  ConcurQuickSort 15
 * 
 *                              36 11 70 4 60 59 55 72 72 57 44 18 1 75 45
 * 
 *                              1 4 11 18 36 44 45 55 57 59 60 70 72 72 75
 * 
 *                              At most 21 threads active at the same time.
 * ---------------------------------------------------------------------------
 */
