/**
 * ===========================================================================
 *                    DEPTH FIRST VISIT OF A TREE
 *                      Graphical User Iterface
 * Grammar (*) for the input tree:  
 *                           char ::= '0' | '1' | ... | '9'
 *                           tree ::= char | - char tree | (tree char tree)
 * Internal tree:            tree = Num  |  Num x tree  |  tree x Num x tree
 *                                 (Leaf)     (Unary)          (Binary)
 * Grammar for the output tree: tree ::= n | (n.tree) | (tree.n.tree)
 *
 * A tree is input as a string generated by the grammar (*) and then 
 * it is visited in the depth first manner.
 * This program uses the class DepthFirstVisit in the file 
 * DepthFirstVisit.java
 * @author Corrado Di Pietro. Modifications by Alberto Pettorossi.
 * ===========================================================================
 */
package parser;
import java.io.IOException;
import javax.swing.JFrame;
import parserUtil.Node;

public class DepthFirstVisitGUI extends JFrame {
    /** Creates new form DepthFirstVisitGUI */
    public DepthFirstVisitGUI() {
        initComponents();
    } 
    /** This method initComponents() is called from within the constructor to
     *  initialize the form.
     *  WARNING: Do NOT modify this code. The content of this method is
     *  always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1   = new javax.swing.JLabel();
        jLabel2   = new javax.swing.JLabel();
        jLabel3   = new javax.swing.JLabel();
        jLabel4   = new javax.swing.JLabel();
        txInput   = new javax.swing.JTextField();
        txOutTree = new javax.swing.JTextField();
        txOutDPF  = new javax.swing.JTextField();
        btParse   = new javax.swing.JButton();
        menuBar   = new javax.swing.JMenuBar();
        fileMenu  = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();
        
        getContentPane().setLayout(null);
        setTitle("Depth First Visit of a Tree");
        setFont(new java.awt.Font("Default", 0, 10));
        setLocationRelativeTo(null);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
// -----------------------------------------------------------
        jLabel1.setBackground(new java.awt.Color(204, 204, 0));
        jLabel1.setFont(new java.awt.Font("Default", 0, 10));
        jLabel1.setText("<HTML>\n<PRE>\n"
        +" Grammar for the input tree:  char ::= '0' | '1' | ... | '9'\n"
        +"                tree ::= char | - char tree | "
        +" (tree char tree)\n"
        +" Internal tree: tree = Num  |  Num x tree  | tree x Num x tree\n"
        +"                      (Leaf)     (Unary)          (Binary)\n"
        +" Grammar for the output tree:\n"
        +"                tree ::= n | (n.tree) | (tree.n.tree)\n"
        +"<PRE>\n</HTML>");

        jLabel1.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLabel1.setOpaque(true);
        getContentPane().add(jLabel1);
        jLabel1.setBounds(5, 5, 425, 95);
// -----------------------------------------------------------
        jLabel2.setText("Input Tree:");
        getContentPane().add(jLabel2);
        jLabel2.setBounds(10, 104, 90, 16);     
  
        getContentPane().add(txInput);
        txInput.setBounds(120, 104, 310, 20);   
// -----------------------------------------------------------
        jLabel3.setText("Output Tree:");
        getContentPane().add(jLabel3);
        jLabel3.setBounds(10, 144, 90, 16);     

        txOutTree.setBackground(new java.awt.Color(204, 255, 204));
        getContentPane().add(txOutTree);
        txOutTree.setBounds(120, 144, 310, 20); 
// -----------------------------------------------------------
        jLabel4.setText("Depth First Visit:");
        getContentPane().add(jLabel4);
        jLabel4.setBounds(10, 184, 140, 16);    

        txOutDPF.setBackground(new java.awt.Color(102, 255, 102));
        getContentPane().add(txOutDPF);
        txOutDPF.setBounds(120, 184, 310, 20);
// -----------------------------------------------------------
        btParse.setText("Parse and Visit");
        btParse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btParseActionPerformed(evt);
            }
        });
        getContentPane().add(btParse);
        btParse.setBounds(160, 220, 138, 26);  
// -----------------------------------------------------------
        fileMenu.setText("File");
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);
        menuBar.add(fileMenu);
        setJMenuBar(menuBar);

        java.awt.Dimension screenSize = 
             java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-444)/2, 
                  (screenSize.height-300)/2, 444, 300); 
    } //GEN-END:initComponents

    private void btParseActionPerformed(java.awt.event.ActionEvent evt) {
        //GEN-FIRST:event_btParseActionPerformed
        // Add your handling code here:
		Node tree = null;
		try {
			tree = DepthFirstVisit.parse(txInput.getText());
		} catch (IOException e) {
			e.printStackTrace();
            txOutTree.setText("IOException!");
        } catch ( ParseException pex ){
            txOutTree.setText(pex.getMessage());
        }
		if (tree != null) {
            txOutTree.setText(tree.toString());
            txOutDPF.setText(DepthFirstVisit.depthFirstVisit(tree));
		}
        
    } //GEN-LAST:event_btParseActionPerformed
    
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
      //GEN-FIRST:event_exitMenuItemActionPerformed
        System.exit(0);
    } //GEN-LAST:event_exitMenuItemActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {
      //GEN-FIRST:event_exitForm
        System.exit(0);
    } //GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new DepthFirstVisitGUI().show();
    }
    
    // Variables declaration - do not modify   //GEN-BEGIN:variables
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JTextField txInput;
    private javax.swing.JTextField txOutTree;
    private javax.swing.JTextField txOutDPF;
    private javax.swing.JButton btParse;
    // End of variables declaration            //GEN-END:variables
}
/** 
 * input:             output:
 * ---------------------------------------------------------------------------
 * javac -classpath . treeparse/DepthFirstVisitGUI.java
 * java  -classpath . treeparse/DepthFirstVisitGUI
 *
 * (-231(-564-78))    (see figures below)
 * ---------------------------------------------------------------------------
 * The string   (-231(-564-78))   encodes the tree:     1
 *                                                     / \
 *                                                     2  4
 *                                                     |  /\
 *                                                     3 5  7
 *                                                       |  |
 *                                                       6  8
 * whose depth first visit is: 3 2 6 5 8 7 4 1
 * ---------------------------------------------------------------------------
 */
