/** 
 * ===========================================================================
 *                              Class Node
 *
 * This class is used for constructing a Node of a binary tree.
 * It can be used for constructing a Node without reference to the
 * parent, or to the left child node, or to the right child node,
 * according to the various constructors.
 * ===========================================================================
 */
package parserUtil;

public class Node {
    public Object  value;    // value field
    public Node    left;     // reference to the left child node
    public Node    right;    // reference to the right child node
    public Node    parent;   // reference to the parent 
   /*
    * Constructor of a Node with:
    * - value v, 
    * - parameters l and r, which are references to the left and right 
    *     child nodes, and
    * - p which is the reference to the parent node. 
    *     This additional parent field is not standard for trees, but
    *     it may speed up some computations over trees.
    */
   public Node( Object v, Node l, Node r, Node p ) {
      if ( v == null )
         throw new IllegalArgumentException( );
      value  = v;
      left   = l;
      right  = r;
      parent = p;
   // updating the parent field of the left child node (if it exists)
   if (l != null) l.parent = this;

   // updating the parent field of the right child node (if it exists)
   if (r != null) r.parent = this;
    }
                                             // Various constructors for:
   public Node( Object v ) {                 // a leaf
      this (v, null, null, null);
   }

   public Node( Object v, Node l ) {         // a unary node
      this (v, l, null, null);
   }
    
   public Node( Object v, Node l, Node r ) { // a binary node
      this (v, l, r, null);
   }

   /**
    * @return the value
    */
   public Object getValue(){
        return value;
   }

   /**
    * @return the left son node
    */
   public Node getLeftChild(){
        return left;
   }

   /**
    * @return the right son node
    */
   public Node getRightChild(){
        return right;
   }

   /**
    * @return the parent, if any
    */
   public Node getParent(){
        return parent;
   }
}
