/**
 * ===========================================================================
 *                                 Class Stack
 *
 * Realization of a stack by using the class Vector. 
 * See the Java Collections API at java.util.*
 *
 * This class Stack resides in the package parserUtil and uses 
 * the class EmptyStackException( String s ) also in the package parserUtil. 
 * ===========================================================================
 */
package parserUtil;
import java.util.Vector;

public class Stack {
    
   Vector stack;

    /**
     * Constructor of the stack.
     */
   public Stack( ) {
      stack = new Vector( );
   }

    /**
     * Pushing an object on the top of the stack.
     * @param x   x is the object to be inserted.
     * @return    the object which is inserted.
     * @exception IllegalArgumentException if the argument is 
     *            <code>null</code>.  
     */
   public Object push( Object x ) {
      if ( x == null )
         throw new IllegalArgumentException( );
         stack.addElement( x ); 
         return x;          // addElement is a method of the class Vector
   }
    
    /**
     * Removing and returning the object at the top the stack.
     * @return    the object on top of the stack.
     * @exception EmptyStackException if the stack is empty.
     */
   public Object pop( ) {
      if ( isEmpty( ) )
         throw new EmptyStackException( "\nError: empty stack\n" );
      return stack.remove( stack.size( ) - 1 );
   }                        // remove is a method of the class Vector

    /**
     * Returning the object at the top of the stack without popping it.
     * (In other languages peek() is also called top()).
     * @return    the object at the top of the stack.
     * @exception EmptyStackException if the stack is empty.
     */
   public Object peek( ) {
      if ( isEmpty( ) )
         throw new EmptyStackException( "\nError: empty stack\n" );
      return stack.lastElement( );
   }                        // lastElement is a method of the class Vector

    /**
     * Checking whether or not the stack is empty.
     * @return  <code>true</code> if the stack is empty;
     *          <code>false</code> otherwise.
     */
   public boolean isEmpty( ) {
      return stack.isEmpty( );  
   }                        // isEmpty is a method of the class Vector
    
    /**
     *  Removing all elements from the stack.
     */
   public void empty( ) {
      stack.clear( );       // clear is a method of the class Vector
   }

}
