/**
 * ===========================================================================
 *             COMPUTING THE i-TH FIBONACCI NUMBER 
 *
 * Thanks to Prof. Vijay K. Garg, Austin, Texas. 
 * With changes by Alberto Pettorossi.
 *
 *    index i:           0, 1, 2, 3, 4, 5, 6,  7, ...
 *    fibonacci number:  0, 1, 1, 2, 3, 5, 8, 13, ...
 *
 *
 * call as:java ConcurExpFibonacci n (with 0 <= n <= 14)
 * Uncomment the lines marked with (***) to see the beginning and the end 
 * of the active threads.
 * (n means: thread begins.   n) means: thread ends.
 * ===========================================================================
 */ 

class ConcurExpFibonacci extends Thread {
   int n;
   int result;

   /** Using the class ConcurExpFibonacci we can construct a thread for 
    *  computing fibonacci(n).
    *  @param n is the argument when we want to compute fibonacci(n).
    */

   public ConcurExpFibonacci(int n){                // constructor
      this.n = n;
   }

   public void run(){
      // setName((new Integer(n)).toString());      // (***)
      // System.out.print("("+ getName() + " " );   // (***)
      if (n==0) {
          result=0;
      }
      else if (n==1) {
          result=1;
      }
      else {
           ConcurExpFibonacci f1 = new ConcurExpFibonacci(n-1);
           ConcurExpFibonacci f2 = new ConcurExpFibonacci(n-2);
           f1.start();
           f2.start();
           try {
               f1.join(); f2.join();
           } catch (InterruptedException e){}
           result = f1.getResult() + f2.getResult();
      };
      // System.out.print( getName() + ") ");       // (***)
   }

   public int getResult() {
      return result;
   }

   public static void main(String [] args){
      int arg =Integer.parseInt(args[0]);
      ConcurExpFibonacci f = new ConcurExpFibonacci(arg);
      f.start();
      try {
          f.join();
      } catch (InterruptedException e){};
      System.out.println("\nfib("+ arg +") = " + f.getResult());
   }
}
/**
 * input:                output:
 * ---------------------------------------------------------------------------
 * javac ConcurExpFibonacci.java
 * java  ConcurExpFibonacci 12  
 *
 *                       fib(12) = 144
 * ---------------------------------------------------------------------------
 * If we uncomment the lines marked with (***) we get:
 * java  ConcurExpFibonacci 4
 *                       (4 (3 (2 (2 (1 1) (1 1) (0 0) 2) (1 1) (0 0) 2) 3) 4)
 *                       fib(4) = 3
 * ---------------------------------------------------------------------------
 * The above string:     (4 (3 (2 (2 (1 1) (1 1) (0 0) 2) (1 1) (0 0) 2) 3) 4)
 * shows that the Java scheduler makes the tree of the recursive calls of 
 * the Fibonacci function (see below) to be computed by the active threads 
 * in a breadth-first manner. Here is the tree of the recursive calls of 
 * the Fibonacci function when we want to compute Fibonacci(4)
 * (for all n >=0, fib(n) denotes Fibonacci(n)):
 *   
 *                                    fib(4)
 *                                  /        \   
 *                             fib(3)         fib(2)
 *                            /      \       /      \
 *                          fib(2) fib(1)  fib(1) fib(0)    
 *                         /      \
 *                       fib(1) fib(0)
 *
 * In our encoding of the active threads via a string, we have adopted 
 * the following convention:
 *         (n  means: "a thread computing fib(n) begins" and 
 *          n) means: "a thread computing fib(n) ends".
 * The maximum number of threads which are active at the same time depends on
 * the shape and the depth of the tree of the recursive calls.
 * ---------------------------------------------------------------------------
 */
