/**
 * ===========================================================================
 *             COMPUTING THE i-TH FIBONACCI NUMBER WITH MEMOIZATION
 * Thanks to Prof. Marco Danelutto. With changes by Alberto Pettorossi.
 *
 * fibArray[i] contains the i-th Fibonacci number.
 *
 *                         index i: 0, 1, 2, 3, 4, 5, 6,  7, ...
 * fibonacci number in fibArray[i]: 0, 1, 1, 2, 3, 5, 8, 13, ...
 *    
 * call as: java ConcurLinearFibonacci n (with n >= 0)
 * ===========================================================================
 */

public class ConcurLinearFibonacci {
//                                               <----- insertion point (<<<)
//                 At this point we will insert the two classes FibThread 
//                 and ArrayClass (see program ConcurLinearFibonacciAll.java)
  /**
   * This class has the main method only.
   * When computing fibonacci(n), for n >= 0, the method main:
   * - constructs the array fibArray of n+1 elements, and 
   * - constructs and starts n+1 threads each of which reads and writes
   *   the elements of fibArray.
   */

    public static void main(String [] args) {
        int n;

        try {
             n = Integer.parseInt(args[0]);
        } catch (Exception e) {
          System.out.println("Please, type-in: java ConcurLinearFibonacci n");
          return;
        }

        System.out.println("Constructing an array of "+(n+1)+" elements.");
        ArrayClass fibArray = new ArrayClass(n+1);
        // each element of the array fibArray is initialized to -1. 

        System.out.println("Constructing and starting "+(n+1)+" threads.");
        Thread[] t = new Thread[n+1];

         // starting the threads in the reverse order to show synchronization.
         // thread t[i] computes fibonacci(i),  for i = 0,...,n.
         for(int i=n; i>=0; i--) {
             t[i] = new FibThread(fibArray, i);
             t[i].start();
            };
       
        System.out.println("fib("+ n +") = " + fibArray.getValue(n));
        System.out.println("\nfibArray:");
        fibArray.printar();
    }
}

/** input:                      output:
 * ---------------------------------------------------------------------------
 *  javac ConcurLinearFibonacci.java   
 *  java  ConcurLinearFibonacci 12
 *                              Constructing an array of 13 elements.
 *                              Constructing and starting 13 threads.
 *                              fib(12) = 144
 *                              
 *                              fibArray:
 *                              0 1 1 2 3 5 8 13 21 34 55 89 144
 * ---------------------------------------------------------------------------
 */
