/**
 * ===========================================================================
 *             COMPUTING THE i-TH FIBONACCI NUMBER WITH MEMOIZATION
 *                    Version with one public class only.
 *
 * Thanks to Prof. Marco Danelutto. With changes by Alberto Pettorossi.
 *
 * fibArray[i] contains the i-th Fibonacci number.
 *
 *    index i:           0, 1, 2, 3, 4, 5, 6,  7, ...
 *    fibonacci number:  0, 1, 1, 2, 3, 5, 8, 13, ...
 *
 * call as: java ConcurLinearFibonacciAll n (with n >= 0)
 *
 * This program ConcurLinearFibonacciAll.java differs from 
 * the program ConcurLinearFibonacci with the three files
 * ConcurLinearFibonacci.java, FibThread.java, and ArrayClass.java,
 * in that: 
 * - the classes FibThread and ArrayClass have been changed from "public" to 
 *   "private static", 
 * - they have been inserted as "inner classes" at the insertion point (<<<) 
 *   of the program ConcurLinearFibonacci.java, and
 * - most comments of the files ConcurLinearFibonacci.java, FibThread.java, 
 *   and ArrayClass.java have been removed for reasons of brevity.
 * ===========================================================================
 */
public class ConcurLinearFibonacciAll {
//                            <----- this is the insertion point (<<<) 
//                                   of the program ConcurLinearFibonacci.java
//     At this point we have inserted the two classes FibThread and ArrayClass
//----------------------------------------------------------------------------
private static class FibThread extends Thread {  

                                           // private data fields:
    private static ArrayClass memo;        //          static array memo
    private int ind;                       //          ind
                                        
    FibThread(ArrayClass arrA, int i) {    // constructor 
        this.memo = arrA;                  // initialization of memo 
        this.ind = i;                      // initialization of ind
    }

    public void run() {
        // setName((new Integer(ind)).toString());               //  (1)
        // System.out.println("Thread "+ getName() + ": begin"); //  (2)
        if (ind == 0) {
             memo.setValue(0,0);
        } else if (ind == 1) {
             memo.setValue(1,1);
        } else {
             int value = memo.getValue(ind-2) + memo.getValue(ind-1);
             memo.setValue(ind,value);
        }
        // System.out.println("Thread "+ getName() + ": end");   //  (3)  
        return;
    }
}

//--------------------------------------------------------------------------
private static class ArrayClass {
  
    private static int [] arrayA;  // data fields: arrayA, dim
    private static int dim;
                                      
    ArrayClass(int d) {            // constructor
        this.arrayA = new int [d]; 
        for(int i=0; i<d; i++) {   // initialization of arrayA 
          this.arrayA[i] = -1;     // all elements of arrayA are set to -1     




          };
        this.dim = d;              // initialization of dim   
    }

    private static void printar() {
        for (int k=0; k<dim; k++) {
            System.out.print(arrayA[k] + " ");
        }
        System.out.println();
    }

    private synchronized void setValue(int index, int value) {
     // System.out.println("New fibonacci number ["+ index +"] = "+value); //
        arrayA[index] = value;
        notifyAll();
     // printar(); // each time an element is added the arrayA is printed  //
        return;
    }

    private synchronized int getValue(int index) {
     // System.out.println("fib(" + index +")=" + arrayA[index]);          //
        while (arrayA[index] == -1) {
            try {
                wait();
            } catch (InterruptedException e) {}
        }
        return(arrayA[index]);
    }
}

//---------------------------------------------------------------------------
    public static void main(String [] args) {
        int n;

        try {
             n = Integer.parseInt(args[0]);
        } catch (Exception e) {
          System.out.println("Please, type-in: java ConcurLinearFibonacci n");
          return;
        }

        System.out.println("Constructing an array of "+(n+1)+" elements.");
        ArrayClass fibArray = new ArrayClass(n+1); 
        // each element of the array fibArray is initialized to -1. 

        System.out.println("Constructing and starting "+(n+1)+" threads.");
        Thread[] t = new Thread[n+1];

         // starting the threads in the reverse order to show synchronization.
         // thread t[i] computes fibonacci(i),  for i = 0,...,n.
         for(int i=n; i>=0; i--) {
             t[i] = new FibThread(fibArray, i);  
             t[i].start();
            };
       
        System.out.println("fib("+ n +") = " + fibArray.getValue(n));
        System.out.println("\nfibArray:");
        fibArray.printar();
    }
}
/** input:                           output:
 * ---------------------------------------------------------------------------
 *  javac ConcurLinearFibonacciAll.java   
 *  java  ConcurLinearFibonacciAll 12
 *                                   Constructing an array of 13 elements.
 *                                   Constructing and starting 13 threads.
 *                                   fib(12) = 144
 *                                   
 *                                   fibArray:
 *                                   0 1 1 2 3 5 8 13 21 34 55 89 144
 * ---------------------------------------------------------------------------
 */
