/**
 * ===========================================================================
 *                       PALINDROME TESTER
 *
 * This program illustrates various classes and methods.
 * - The class InputStreamReader:
 *           for transforming a stream of bytes into a stream of characters.
 * - The class BufferedReader:
 *           for transforming a stream of characters into a line.
 *   These two classes are required when reading the input from the keyboard.
 * - The class StringTokenizer: x.hasMoreTokens(), x.nextToken().
 * - The method Integer.parseInt(string n):  
 *           for transforming a string into an int.
 * - The method x.equals(y)
 * - The class ArrayList which is a collection of elements with three methods:
 *       size(), get(i), and add(x).
 * - The "wrapper class" Integer: 
 *       for transforming an int element into an Integer object.
 * 
 * We get a different, correct program if we comment lines (1) 
 * and we uncomment lines (2).
 *
 * With lines (1) uncommented and lines (2) commented we get a program 
 * which constructs the object palinTester of the class PalindromeTester.
 *
 * With lines (1) commented and lines (2) uncommented we get a program 
 * which does NOT construct any object of the class PalindromeTester,
 * and the computation is performed by the palindrome method, 
 * which is a static method.
 * ===========================================================================
 */
import java.util.*;
import java.io.*;

public class PalindromeTester {

   /** Using the class PalindromeTester we can construct an object 
    *  which has a method palindrome. This method takes an array which 
    *  holds objects of the class Integer, and checks whether or not 
    *  that array is palindrome. An array ns is said to be palindrome iff 
    *  for any i within bounds,  ns.get(i).equals(ns.get(size - i - 1)).   
    *  @param ns - it is the array which holds objects of the class Integer.
    */
  public boolean palindrome(ArrayList ns) {           // a method     // (1)
  // static public boolean palindrome(ArrayList ns) { // a method     // (2)
    boolean answer = true;  
    int     size   = ns.size();
    int     middle = size / 2;       

    for (int i = 0; i < middle; i++) {
      answer = ns.get(i).equals(ns.get(size - i - 1)) && answer;
      };

    return answer;
  }

 public static void main(String[] args) {
   String inputLine = "";
   ArrayList numbers = new ArrayList();
   String number; 
   int m;
   Integer n;        // wrapper for int. n is an object of the class Integer
   InputStreamReader byteInput = new InputStreamReader(System.in);
   BufferedReader    input     = new BufferedReader(byteInput);
   try {
         inputLine = input.readLine();    // readLine() requires a try-catch
       } catch (java.io.IOException e) {
         System.out.println("Error in input.read()");
       };

   StringTokenizer tokensInputLine = new StringTokenizer(inputLine," ");
      
   while (tokensInputLine.hasMoreTokens()) {
      number = tokensInputLine.nextToken();
      m = Integer.parseInt(number);
      n = new Integer(m);
      numbers.add(n);
   };
   
   System.out.print("\nThe sequence of integers:\n\" ");
   for (int i = 0; i < numbers.size(); i++) {
      System.out.print(numbers.get(i) + " ");
   }
   System.out.println("\"");
   
   PalindromeTester palinTester = new PalindromeTester(); // (1) 
   //                                                     // (2): empty line
   if (palinTester.palindrome(numbers)) {                 // (1)
   //if (palindrome(numbers)) {                           // (2)
      System.out.println("IS palindrome."); 
      }
   else {
      System.out.println("IS NOT palindrome.");
   };
   }
}
/**
 * input:                output:
 * ---------------------------------------------------------------------------
 * javac Palindrome.java
 * java  Palindrome
 * 3    4 5     4   3
 *                       The sequence of integers:
 *                       " 3 4 5 4 3 "
 *                       IS palindrome.
 * java  Palindrome
 * 3  5  4
 *                       The sequence of integers:
 *                       " 3 5 4 "
 *                       IS NOT palindrome.
 * ---------------------------------------------------------------------------
 */
