/**
 * ===========================================================================
 *           Planet.java : CONSTRUCTING AND CHANGING OBJECTS
 *
 * Parameters are passed in the call-by-value mode:
 * We can change the fields of objects, not their references.
 *
 * Every static field (such as countOfPlanets) is shared among all objects of 
 * the class. Every static field is the same for all objects of the class
 * during the whole computation.
 * ===========================================================================
 */
public class Planet {
    public int idNum = 0;                // idNum should not be static
    public String name = "";             // name  should not be static 
                   // countOfPlanets is the same for all objects of the class.
    private static int countOfPlanets = 0; 

    public Planet(String planetName) {   // constructor
       this.countOfPlanets++;            // every planet is constructed 
       this.idNum = countOfPlanets;      // with a different idNum
       this.name  = planetName;    
    } 

    public void setName(String newname) {
       this.name = newname;
    }  

 public static void main(String [] args) {
    Planet mercury = new Planet("Mercury");
    Planet venus   = new Planet("Venus");
    System.out.println("Planet " + mercury.name + ", idNum: " +mercury.idNum);
    System.out.println("Planet " + venus.name   + ", idNum: " + venus.idNum);

                   // Changing a field of an object via a method or 
                   // the field name. 
    mercury.setName("Mercurio");
    venus.idNum--; // It is ok for Venus to have the same idNum as Mercurio.

    System.out.println("Planet " + mercury.name + ", idNum: " +mercury.idNum);
    System.out.println("Planet " + venus.name + ", idNum: " + venus.idNum);
    System.out.println("Existing Planets: " + countOfPlanets);
    }
}
/**
 * input:                      output:
 * ---------------------------------------------------------------------------
 * javac Planet.java
 * java  Planet
 *                             Planet Mercury, idNum: 1
 *                             Planet Venus, idNum: 2
 *                             Planet Mercurio, idNum: 1
 *                             Planet Venus, idNum: 1
 *                             Existing Planets: 2
 * ---------------------------------------------------------------------------
 */
