/**
 * ===========================================================================
 *     SORTING AND SEARCHING ARRAYS BY USING JAVA METHODS OF java.util.*
 *
 * The main method first randomly generates an array (as an ArrayList) of 
 * n elements and then sorts it. The value of n is given in input to the
 * method main as an argument of the command line: 
 * java ArrayListSort n  (n should be at least 1).
 *
 * Then the main method transforms the generated ArrayList into an Object []
 * and it sorts the Object [] array which is obtained, by using the Java 
 * method Arrays.sort(Object [] array). The sorting algorithm used by that 
 * method is a variant of quicksort.
 *
 * The  main method also checks whether or not a randomly chosen element elem
 * occurs in the array by using the Java method 
 * Arrays.binarySearch(Object [] array, Object elem). 
 *
 * The program illustrates the use of iterators, ArrayList, Object [], int [],
 * and List, and the various conversions among these types.
 * The program also illustrates the use of the Java method 
 *    Arrays.sort(Object [] a, Comparator c)
 * which makes use of a comparator.
 * 
 * All elements of the array should be comparable according to the 
 * "natural ordering", or, in the case we use a comparator, according to
 * the specified comparator. 
 * ===========================================================================
 */
/** In order to import: 
 *  java.util.Random, java.util.ArrayList, java.util.Iterator,
 *  java.util.Arrays, java.util.Collections, java.util.Comparator, and 
 *  java.util.List, using one command only we simply type: import java.util.*
 */
import java.util.*;

public class ArrayListSort {

// ---------------------------------------------------------------------------
 public static void main (String[] args) {

        // length of the array to sort
        int  n; 
        try {
             n = Integer.parseInt(args[0]);
        } catch (Exception e) {
        System.out.print("\nType: java ArrayListSort n (with n >= 1). ");
        System.out.print("Now we have assumed n = 20.\n");
             n = 20;
        }

        // random numbers in {0,..., MAX-1}
        final int  MAX = 80; 

        ArrayList arrListA = new ArrayList();

        // initializing the ArrayList arrListA with Integers  
        Random generator = new Random();
        for(int i=0; i<n; i++) {
            arrListA.add(new Integer(generator.nextInt(MAX)));
            }
        // printing the initial ArrayList arrListA
        // use of the iterator iter for the ArrayList arrListA
        System.out.println("Initial array as ArrayList:");
        Iterator iter = arrListA.iterator();
        while (iter.hasNext()) {
           System.out.print((iter.next()).toString() + " ");
        };
        System.out.println();
        // ----------------------------------------------------- begin -----//
        // from ArrayList() to Object []                                    //
        Object [] arrayB = arrListA.toArray();                              //
                                                                            //
        // sorting the Object [] arrayB using the Arrays.sort method        //
        Arrays.sort(arrayB);                                                //
        // printing the sorted array                                        //
        System.out.println("Sorted array:");                                //
        for (int i=0; i<n; i++)                                             //
           System.out.print(arrayB[i].toString() + " ");                    //
        System.out.println();                                               //
                                                                            //
        // elem is the Integer to be found in the Object [] arrayB          //
        Integer elem = new Integer(generator.nextInt(MAX));                 //
        // ----------------------------------------------------- end -------//
/**
 * ---------------------------------------------------------------------------
 * The statements between the --- begin and --- end lines above,
 * can be replaced by the following ones, where we perform a transformation
 * from ArrayList arrListA to int [] arrayB.
 * The int [] arrayB is then sorted by using Arrays.sort(int [] arrayB).
 * ---------------------------------------------------------------------------
 */  
/**                                                                       
// ------------------------------------------------------------- begin1 ----//
        // transformation from ArrayList arrListA to int [] arrayB          //
        int [] arrayB = new int [n];                                        //
        System.out.println("Initial array as int []:");                     //
        for (int i=0; i<n; i++) {                                           //
           arrayB[i] = Integer.parseInt(arrListA.get(i).toString());        //
           System.out.print(arrayB[i] + " ");                               //
        };                                                                  //
        System.out.println();                                               //
                                                                            //
        // sorting the int [] arrayB using the Arrays.sort method           //
        Arrays.sort(arrayB);                                                //
        // printing the sorted array                                        //
        System.out.println("Sorted array:");                                //
        for (int i=0; i<n; i++)                                             //
           System.out.print(arrayB[i] + " ");                               //
        System.out.println();                                               //
                                                                            //
        // elem is the int to be found in the Object [] arrayB              //
        int elem = generator.nextInt(MAX);                                  //
// ------------------------------------------------------------- end1 ------//
*/
        /** Binary Search in the ordered array.
         *  ip is an index in {0,...,n-1}, such that arrayB[ip] = elem.
         *  If elem does not occur in arrayB then ip is undefined.
         */
        int ip = Arrays.binarySearch(arrayB, elem);
        if (ip >= 0) 
          { System.out.println("An integer " + elem +" is in the array " +
                               "at position " + ip + ".");}
        else 
          { System.out.println("The integer "+ elem +" is NOT in the " +
                               "array.");};
// -------------------------------------------------------------------------//
        // new comparator for the elements of the given ArrayList arrListA:
        //    n1 < n2 iff (n1 mod 10) < (n2 mod 10)

        Comparator comp = new Comparator () {
               public int compare(Object obj1, Object obj2) {
                  int n1 = new Integer(obj1.toString()).intValue();
                  int n2 = new Integer(obj2.toString()).intValue();
                  return (n1 % 10) - (n2 % 10);
               };
        };

        // In order to use the comparator we need to view arrListA as a List,
        // not as an ArrayList.
        // The following statement sorts (List)arrListA using the
        // Collections.sort method and the comparator comp.
        Collections.sort((List)arrListA, comp);  

        System.out.println("\nArray sorted according to the modulo " +
                           "comparator:");
        Iterator iter1 = arrListA.iterator();
        while (iter1.hasNext()) {
           System.out.print((iter1.next()).toString() + " ");
        };
        System.out.println();
 }
}
/** 
 * input:                output:
 * ---------------------------------------------------------------------------
 * javac ArrayListSort.java
 * java  ArrayListSort 18
 *                       Initial array as ArrayList:
 *                       64 51 28 50 47 4 43 21 70 33 39 21 10 32 22 31 22 63
 *                       Sorted array:
 *                       4 10 21 21 22 22 28 31 32 33 39 43 47 50 51 63 64 70
 *                       An integer 22 is in the array at position 4.
 *
 *                       Array sorted according to the modulo comparator:
 *                       50 70 10 51 21 21 31 32 22 22 43 33 63 64 4 47 28 39
 * ---------------------------------------------------------------------------
 */
