/**
 * ===========================================================================
 *                            BUBBLE SORT
 *
 * The main method randomly generates an array of n elements and then 
 * sorts it. The value of n is given in input to the method main
 * as an argument of the command line: java BubbleSort n
 * n should be at least 1.
 * 
 * The class bubbleSort uses bubble sort for sorting.
 * The array is an array of OrderedInteger elements, and 
 * OrderedInteger is a class which implements the interface Comparable 
 * (see the interface Comparable and its method: compareTo).
 *
 * This program illustrates the use of:
 * - interfaces,  and
 * - inner classes. 
 *
 * Thanks to G. Pighizzini and M. Ferrari. 
 * ===========================================================================
 */
import java.util.Random;

public class BubbleSort {

/** 
 * ---------------------------------------------------------------------------
 * This method bubbleSort sorts an array by using the bubble sort technique.
 * Notice the object-oriented view in the expression: 
 *
 *    arrayA[i].compareTo(arrayA[i+1])
 * 
 * Indeed, the binary comparison is performed by making the object arrayA[i] 
 * to activate its method compareTo which takes the other object as  
 * an argument.
 * ---------------------------------------------------------------------------
 */

  public static void bubbleSort(OrderedInteger [] arrayA) { 

      int i; 
      boolean modified;
      do {
         i=0;
         modified = false;
         while (i < ((arrayA.length)-1)) 
              { if (arrayA[i].compareTo(arrayA[i+1]) > 0) {
                                           // swapping the elements
                   swap (arrayA[i], arrayA[i+1]);
                   modified = true; 
                   };
                i++;       
              };
         }
      while (modified);
   }

   public static void swap (OrderedInteger a, OrderedInteger b) {
      OrderedInteger temp = new OrderedInteger(a.getValue());
      a.setValue(b.getValue()); 
      b.setValue(temp.getValue()); 
   };
// ---------------------------------------------------------------------------
// This class OrderedInteger is an "inner class".

   public static class OrderedInteger implements Comparable {

        int n;                             // field  

        public OrderedInteger(int n) {     // constructor
        this.n = n;
        }
    
        public int getValue () {           // the method getValue ()
          return this.n;
        }

        public void setValue (int m) {     // the method setValue ()
          this.n = m;
        }

        /**
         * Here is the method compareTo (Object obj).
         * Notice that its first line has exactly the same signature of the 
         * method compareTo(Object obj) of 
         * the interface Comparable.
         */

        public int compareTo (Object obj) { 
           if (this.getValue() <  (((OrderedInteger)obj).getValue()))
                {return -1;} else 
           if (this.getValue() == (((OrderedInteger)obj).getValue()))
                {return 0;} 
           else {return 1;}
        }

        public String toString () {        // the method toString ()
          return Integer.toString(this.n);  
        }                                 
        /**
         * In the expression Integer.toString(this.n) this.n has type int. 
         * Equivalently, in the method toString () we could have written the 
         * statement: return "" + this.n;
         */ 
    }; 

// ---------------------------------------------------------------------------
 public static void main (String[] args) {

        int  n;                       // length of the array to sort

        try {
             n = Integer.parseInt(args[0]);
        } catch (Exception e) {
        System.out.print("\nType: java ConcurQuickSort n (with n >= 1). ");
        System.out.print("Now we have assumed n = 20.\n");
             n = 20;
        }

        final int  MAX = 80;          // random numbers in {0,..., MAX-1}

        OrderedInteger [] arrayA = new OrderedInteger [n];

                                      // initializing the array of numbers
        Random generator = new Random();
        for(int i=0; i<n; i++) {
            arrayA[i] = new OrderedInteger(generator.nextInt(MAX));
            }
                                      // printing the initial array
        // notice the for statement whose body is one statement only
        System.out.println("Initial array:");
        for (int i=0; i<n; i++) System.out.print(arrayA[i].toString() + " ");
        System.out.println();

                                      // invoking the method bubbleSort
        BubbleSort.bubbleSort(arrayA);

                                      // printing the sorted array
        System.out.println("Sorted array:"); 
        for (int i=0; i<n; i++) System.out.print(arrayA[i].toString() + " ");
        System.out.println();
    }

}
/**
 * Notice that System.out.print(...) and System.out.println(...) automatically
 * invoke toString(), if a non-string object is supplied.
 * We could have written: arrayA[i], instead of arrayA[i].toString()
 */
/** 
 * input:                output:
 * ---------------------------------------------------------------------------
 * javac BubbleSort.java
 * java  BubbleSort 18
 *                       Initial array:
 *                       64 76 12 76 55 35 57 71 71 13 39 14 6 55 70 58 9 74
 *                       Sorted array:
 *                       6 9 12 13 14 35 39 55 55 57 58 64 70 71 71 74 76 76
 * ---------------------------------------------------------------------------
 */
